<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Pinpoint\CarouselManager\Model;

use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Api\ExtensibleDataObjectConverter;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Store\Model\StoreManagerInterface;
use Pinpoint\CarouselManager\Api\Data\SlideCtaInterfaceFactory;
use Pinpoint\CarouselManager\Api\Data\SlideCtaSearchResultsInterfaceFactory;
use Pinpoint\CarouselManager\Api\SlideCtaRepositoryInterface;
use Pinpoint\CarouselManager\Model\ResourceModel\SlideCta as ResourceSlideCta;
use Pinpoint\CarouselManager\Model\ResourceModel\SlideCta\CollectionFactory as SlideCtaCollectionFactory;

class SlideCtaRepository implements SlideCtaRepositoryInterface
{

    protected $resource;

    protected $slideCtaFactory;

    protected $slideCtaCollectionFactory;

    protected $searchResultsFactory;

    protected $dataObjectHelper;

    protected $dataObjectProcessor;

    protected $dataSlideCtaFactory;

    protected $extensionAttributesJoinProcessor;

    private $storeManager;

    private $collectionProcessor;

    protected $extensibleDataObjectConverter;

    /**
     * @param ResourceSlideCta $resource
     * @param SlideCtaFactory $slideCtaFactory
     * @param SlideCtaInterfaceFactory $dataSlideCtaFactory
     * @param SlideCtaCollectionFactory $slideCtaCollectionFactory
     * @param SlideCtaSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param ExtensibleDataObjectConverter $extensibleDataObjectConverter
     */
    public function __construct(
        ResourceSlideCta $resource,
        SlideCtaFactory $slideCtaFactory,
        SlideCtaInterfaceFactory $dataSlideCtaFactory,
        SlideCtaCollectionFactory $slideCtaCollectionFactory,
        SlideCtaSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource = $resource;
        $this->slideCtaFactory = $slideCtaFactory;
        $this->slideCtaCollectionFactory = $slideCtaCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataSlideCtaFactory = $dataSlideCtaFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
        $this->collectionProcessor = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \Pinpoint\CarouselManager\Api\Data\SlideCtaInterface $slideCta
    ) {
        /* if (empty($slideCta->getStoreId())) {
            $storeId = $this->storeManager->getStore()->getId();
            $slideCta->setStoreId($storeId);
        } */
        
        $slideCtaData = $this->extensibleDataObjectConverter->toNestedArray(
            $slideCta,
            [],
            \Pinpoint\CarouselManager\Api\Data\SlideCtaInterface::class
        );
        
        $slideCtaModel = $this->slideCtaFactory->create()->setData($slideCtaData);
        
        try {
            $this->resource->save($slideCtaModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the slideCta: %1',
                $exception->getMessage()
            ));
        }
        return $slideCtaModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function get($slideCtaId)
    {
        $slideCta = $this->slideCtaFactory->create();
        $this->resource->load($slideCta, $slideCtaId);
        if (!$slideCta->getId()) {
            throw new NoSuchEntityException(__('SlideCta with id "%1" does not exist.', $slideCtaId));
        }
        return $slideCta->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->slideCtaCollectionFactory->create();
        
        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \Pinpoint\CarouselManager\Api\Data\SlideCtaInterface::class
        );
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \Pinpoint\CarouselManager\Api\Data\SlideCtaInterface $slideCta
    ) {
        try {
            $slideCtaModel = $this->slideCtaFactory->create();
            $this->resource->load($slideCtaModel, $slideCta->getSlidectaId());
            $this->resource->delete($slideCtaModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the SlideCta: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($slideCtaId)
    {
        return $this->delete($this->get($slideCtaId));
    }

    public function getBySlideId($slideId)
    {
        return $this->slideCtaCollectionFactory->create()->addFieldToFilter('slide_id', $slideId)
            ->setOrder('sort_order', 'asc');
    }
}